from logging.handlers import RotatingFileHandler
import logging
from typing import Union, Optional
from pathlib import Path
import os


def get_logger(name: str,
               level: int = logging.INFO,
               max_bytes: int = 20_000_000,  # 20 MB
               backup_count: int = 3,
               custom_log_file: Optional[Union[str, Path]] = None,
               format='[%(process)d][%(filename)s][%(levelname)s]%(asctime)s: %(message)s'):
    # retreive singleton logger by name.
    logger = logging.getLogger(name)
    # if this is not the first call, the {name} logger will already have handlers.
    if logger.hasHandlers():
        return logger
    # logging.INFO == 20, logging.DEBUG == 10
    logger.setLevel(int(os.environ.get('LOGLEVEL', level)))
    # create log directory if it doesn't currently exist.
    log_file = Path(custom_log_file) if custom_log_file is not None else Path.home(
    ).joinpath(f'{name}_logs/{name}.log')
    log_file.parent.mkdir(exist_ok=True, parents=True)
    # add file handler.
    file_handler = RotatingFileHandler(log_file,
                                       maxBytes=max_bytes,
                                       backupCount=backup_count)
    formatter = logging.Formatter(format)
    file_handler.setFormatter(formatter)
    logger.addHandler(file_handler)
    sh = logging.StreamHandler()
    sh.setFormatter(formatter)
    logger.addHandler(sh)
    logger.propagate = False
    return logger
